indexing
	description: "Objects that execute features of some class"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

deferred class
	CLASS_TESTER

feature -- Basic operations
	execute_small_suite is
			-- Execute small test suite
		deferred
		end

	execute_medium_suite is
			-- Execute medium test suite
		deferred
		end

	execute_large_suite is
			-- Execute large test suite
		deferred
		end

feature -- Integer routines
	procedure_1 (param, x: INTEGER) is
			-- Do nothing
		do
		end

	procedure_2 (param, x: INTEGER) is
			-- Print something
		do
			print (param + x)
		end

	function (param, x: INTEGER): INTEGER is
			-- Some integer value
		do
			Result := (param + x).item
		end

	predicate_1 (param, x: INTEGER): BOOLEAN is
			-- Some boolean value
		do
			Result := function (param, x) \\ 2 = 0
		end

	predicate_2 (param, x: INTEGER): BOOLEAN is
			-- Some boolean value
		do
			Result := param < x
		end

	predicate_3 (x: INTEGER_32_REF; param: INTEGER): BOOLEAN is
			-- Some boolean value
		do
			Result := param < x
		end

feature {NONE} -- Implementation
	random: MY_RANDOM is
			-- Random generator
		once
			create Result.make
		end

	item (str: STRING): CHARACTER is
			-- Random character from `str'
		require
			str_exists: str /= Void
			str_nonempty: not str.is_empty
		local
			i: INTEGER
		do
			i := random.bounded_item (1, str.count)
			random.forth
			Result := str.item (i)
		end

	substring (str: STRING): STRING is
			-- Random substring of `str'
		require
			str_exists: str /= Void
		local
			min, max: INTEGER
		do
			if not str.is_empty then
				min := random.bounded_item (1, str.count)
				random.forth
				max := random.bounded_item (min - 1, str.count)
				random.forth
				Result := str.substring (min, max)
			else
				create Result.make_empty
			end
		ensure
			result_exists: Result /= Void
		end

	nonempty_substring (str: STRING): STRING is
			-- Nonempty substring of `str'
		require
			str_exists: str /= Void
			str_nonempty: not str.is_empty
		local
			min, max: INTEGER
		do
			min := random.bounded_item (1, str.count)
			random.forth
			max := random.bounded_item (min, str.count)
			random.forth
			Result := str.substring (min, max)
		ensure
			result_exists: Result /= Void
			result_nonempty: not Result.is_empty
		end

	uniform_string (s: STRING; size_bound: INTEGER): STRING is
			-- String consisting of random number of one of characters from `s'
		require
			size_bound_positive: size_bound > 0
		local
			c: CHARACTER
		do
			c := s.item (random.bounded_item (1, s.count))
			random.forth
			create Result.make_filled (c, random.bounded_item (1, size_bound))
			random.forth
		ensure
			result_not_empty: not Result.is_empty
		end

	text: STRING is "{
Learning all the technical details of inheritance and related mechanisms, as we did in
part C, does not automatically mean that we have fully grasped the methodological
consequences. Of all issues in object technology, none causes as much discussion as the
question of when and how to use inheritance; sweeping opinions abound, for example on
Internet discussion groups, but the literature is relatively poor in precise and useful advice.

In this chapter we will probe further into the meaning of inheritance, not for the sake
of theory, but to make sure we use it best to benefit our software development projects.We
will in particular try to understand how inheritance differs from the other inter-module
relation in object-oriented system structures, its sister and rival, the client relation: when
to use one, when to use the other, when both choices are acceptable. Once we have set the
basic criteria for using inheritance, identifying along the way the typical cases in which
it is wrong to use it, we will be able to devise a classification of the various legitimate
uses, some widely accepted (subtype inheritance), others, such as implementation or
facility inheritance, more controversial. Along the way we will try to learn a little from the
experience in taxonomy, or systematics, gained from older scientific disciplines.

24.1 HOW NOT TO USE INHERITANCE
To arrive at a methodological principle, it is often useful as illustrated by so many other
discussions in this book to study first how not to do things. Understanding a bad idea
helps find good ones, which we might otherwise miss. In too constantly warm a climate, a
pear tree will not flower; it needs the jolt of Winter frost to attain full bloom in the Spring.
Here the jolt is obligingly provided by a widely successful undergraduate textbook,
used throughout the world to teach software engineering to probably more computing
science students than any other. Already in its fourth edition, it introduced some elements of
object orientation, including a discussion of multiple inheritance. Here is the beginning:
Multiple inheritance allows several objects to act as base objects and is supported
in object-oriented languages such as [the notation of the present book] [M 1988].
The bibliographic reference is to the first edition of the present book. Apart from the
unfortunate use of objects for classes, this is an auspicious start. The extract continues:
	}"

	strange_text: STRING is "test	test	test	test	test	test	test	test	test	test	test	%Ntest test test test test test test test test test test %R%N%R%N"
end
