indexing
	description: "Objects that execute features from DS_TOPOLOGICAL_SORTER class"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	DS_TOPOLOGICAL_SORTER_TESTER

inherit
	CLASS_TESTER

feature -- Basic operations
	execute_small_suite is
			-- Execute small test suite
		do
			test1 (3, 4)
		end

	execute_medium_suite is
			-- Execute medium test suite
		do
			test1 (1, 10)
			test2 (3, 10)
			test3 (1, 10)
			test4 (3)
		end

	execute_large_suite is
			-- Execute large test suite
		do
			test1 (30, 10)
			test2 (10, 10)
			test3 (10, 10)
			test4 (30)
		end

feature {NONE} -- Parameters
	item_bound: INTEGER is 100

feature {NONE} -- Tests
	test1 (count, size: INTEGER) is
			-- Test 1
		local
			i, j, n, m: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				m := random.bounded_item (0, size)
				random.forth
				from j := 1 until j > m and j > 1 loop
					create sorter.make_default
					n := random.bounded_item (0, size)
					random.forth
					create sorter.make (n)
					sorter.set_equality_tester (Void)
					j := j + 1
				end
				create_items (m, Void)
				put_all
				put_relations (m)
				execute_all_functions
				from j := 1 until j > m and j > 1 loop
					sorter.sort
					sorter.wipe_out
					random.forth
					j := j + 1
				end
				i := i + 1
			end
		end

	test2 (count, size: INTEGER) is
			-- Test 2 (do everything mulpiple times without recreating sorter)
		local
			i, j, n, m: INTEGER
			last_greatest: INTEGER_REF
		do
			n := random.bounded_item (0, size)
			random.forth
			create sorter.make (n)
			from
				i := 1
			until
				i > count
			loop
				m := random.bounded_item (0, size)
				random.forth
				from j := 1 until j > m and j > 1 loop
					if sorter.is_empty then
						sorter.set_equality_tester (Void)
					end
					j := j + 1
				end
				create_items (m, last_greatest)
				if not items.is_empty then
					last_greatest := items [items.upper]
				end
				put_all
				put_relations (m)
				from j := 1 until j > m and j > 1 loop
					sorter.sort
					j := j + 1
				end
				execute_all_functions
				i := i + 1
			end
		end

	test3 (count, size: INTEGER) is
			-- Test 3 (same as 1, but with equality tester)
		local
			i, j, n, m: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				m := random.bounded_item (0, size)
				random.forth
				from j := 1 until j > m and j > 1 loop
					n := random.bounded_item (0, size)
					random.forth
					create sorter.make_default
					create sorter.make (n)
					sorter.set_equality_tester (create {KL_EQUALITY_TESTER [INTEGER_REF]})
					j := j + 1
				end
				create_items (m, Void)
				put_all
				put_relations (m)
				execute_all_functions
				from j := 1 until j > m and j > 1 loop
					sorter.sort
					sorter.wipe_out
					j := j + 1
				end
				i := i + 1
			end
		end

	test4 (count: INTEGER) is
			-- Test 4 (like test 2, but array is always empty)
		local
			i: INTEGER
		do
			create sorter.make (0)
			from
				i := 1
			until
				i > count
			loop
				sorter.set_equality_tester (create {KL_EQUALITY_TESTER [INTEGER_REF]})
				sorter.sort
				execute_all_functions
				i := i + 1
			end
		end

feature {NONE} -- Implementation
	execute_all_functions is
			-- Execute functions on `sorter'
		local
			b: BOOLEAN
			i: INTEGER_REF
			n: INTEGER
			t: KL_EQUALITY_TESTER [INTEGER_REF]
		do
			t := sorter.equality_tester
			if random.decide (0.5) then
				b := sorter.equality_tester_settable (create {KL_EQUALITY_TESTER [INTEGER_REF]})
			else
				b := sorter.equality_tester_settable (Void)
			end
			random.forth
			b := sorter.is_empty
			b := sorter.is_sorted
			b := sorter.has_cycle
			n := sorter.count
			n := sorter.capacity
			if random.decide (0.9) then
				random.forth
				if random.decide (0.5) or items.is_empty then
					random.forth
					i := random.bounded_item (-item_bound, item_bound).to_reference
				else
					random.forth
					i := items.item (random.bounded_item (items.lower, items.upper))
				end
			end
			random.forth
			b := sorter.has (i)
			n := sorter.index_of (i)
		end

	put_all is
			-- Put all elements from `item' to `sorter' using `put', `force' or `force_relation'
		local
			i: INTEGER
		do
			from
				i := items.lower
			until
				i > items.upper
			loop
				if sorter.count < sorter.capacity then
					if random.decide (0.8) then
						sorter.put (items [i])
					else
						random.forth
						if random.decide (0.5) then
							sorter.force (items [i])
						else
							random.forth
							sorter.force_relation (items [i], items [random.bounded_item (items.lower, i)])
						end
					end
					random.forth
				else
					if random.decide (0.5) then
						sorter.force (items [i])
					else
						random.forth
						sorter.force_relation (items [i], items [random.bounded_item (items.lower, i)])
					end
					random.forth
				end
				i := i + 1
			end
		end

	remove_all is
			-- Remove all items from `sorter'
		local
			i: INTEGER
		do
			from
				i := items.lower
			until
				i > items.upper
			loop
				sorter.remove (items.item (i))
				i := i + 1
			end
		end

	create_items (m: INTEGER; last_greatest: INTEGER_REF) is
			-- Create `items' of size `m' and fill them withh references to succesive integers
		local
			i, l, start: INTEGER
		do
			if last_greatest = Void then
				l := random.bounded_item (- item_bound, item_bound - m)
			else
				l := last_greatest.item + 1
			end
			create items.make (l, l + m - 1)
			start := l
			if not items.is_empty and last_greatest = Void then
				items.put (Void, l)
				start := l + 1
			end
			from
				i := start
			until
				i > l + m - 1
			loop
				items.put (i.to_reference, i)
				i := i + 1
			end
		end

	put_relations (count: INTEGER) is
			-- Put `count' random relations to `sorter'
		local
			i, k, l: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				k := random.bounded_item (items.lower, items.upper)
				random.forth
				l := random.bounded_item (items.lower, items.upper)
				random.forth
				sorter.put_relation (items [k], items [l])
				i := i + 1
			end
		end

	items: ARRAY [INTEGER_REF]

	sorter: DS_TOPOLOGICAL_SORTER [INTEGER_REF]
end
