class
	FRACTION2

inherit
	NUMERIC
		redefine
			out
		end

create
	make

feature -- Creation
	make (a, b: INTEGER)
			-- Creation procedure.
		require
			denominator_not_zero: b /= 0
		do
			n := a
			d := b
			reduce
		end

feature -- Access

	one: like Current
			-- Neutral element for "*" and "/"
		do
			create Result.make (1, 1)
		end

	zero: like Current
			-- Neutral element for "+" and "-"
		do
			create Result.make (0, 1)
		end

feature -- Status report

	divisible (other: like Current): BOOLEAN is
			-- May current object be divided by `other'?
		do
			Result := (other.n /= 0)
		end

	exponentiable (other: NUMERIC): BOOLEAN is
			-- May current object be elevated to the power `other'?
		do
			Result := false -- Roots (that you usually will get with fractions in the power) are real and therefore no representation as (integer-)fraction is possible.
		end

feature -- Basic operations

	infix "+" (other: like Current): like Current is
			-- Sum with `other' (commutative).
		do
			create Result.make (n*other.d + other.n*d, d*other.d)
			Result.reduce
		end

	infix "-" (other: like Current): like Current is
			-- Result of subtracting `other'
		do
			create Result.make (n*other.d - other.n*d, d*other.d)
			Result.reduce
		end

	infix "*" (other: like Current): like Current is
			-- Product by `other'
		do
			create Result.make (n*other.n, d*other.d)
			Result.reduce
		end

	infix "/" (other: like Current): like Current is
			-- Division by `other'
		do
			create Result.make (n*other.d, d*other.n)
			Result.reduce
		end

	prefix "+": like Current is
			-- Unary plus
		do
			create Result.make (n, d)
		end

	prefix "-": like Current is
			-- Unary minus
		do
			create Result.make (-n, d)
			Result.reduce
		end

feature -- Values
	n, d: INTEGER -- Numerator and denominator.

feature -- Reduction
	gcd: INTEGER
			-- Calculate the greatest common divisor of d and n.
		local
			m, l: INTEGER
		do
			if d = 0 then
				Result := n.abs
			elseif n = 0 then
				Result := d.abs
			else
				-- If neither d nor n are equal to 0 calculate the gcd
				from
					m := d.abs
					l := n.abs
				invariant
					m>0
					l>0
				variant
					m.max (l)
				until
					m = l
				loop
					if
						m > l
					then
						m := m - l
					else
						l := l - m
					end
				end
				Result := m
			end
		end

	reduce
			-- Reduce the fraction to the smallest possible numerator and denominator.
		local
			div: INTEGER
		do
			div := gcd
			n := n // div
			d := d // div
			if n < 0 and d < 0 then
				n := n.abs
				d := d.abs
			end
		end

feature -- Output
	out: STRING
			-- Display the fraction.
		do
			Result := (n.out + "/" + d.out)
		end

invariant
	denominator_not_zero: d /= 0
end
