indexing
	description: "Representation of a person and its family relations"
	author: "P.Bucher, N.Ranieri, T.Britschgi"
	date: "$Today$"
	revision: "$1.0$"

class
	PERSON2

-- COMMENTS
--
--  > erneute Heirat adoptiert Kind

-- > replaceMarriageDate is drin wegen dem Ansatz, zuerst ??.??.???? zu setzen.

-- evtl. noch bei allen dates ein require valid date reinmachen? kann man in eiffel static auf ein funktion zugreifen, e.g. valid_date

-- Operator "=" hinzufgen, der die IDs von zwei PERSON Objekten vergleicht andstelle des default "=" das nur die Pointer vergleicht?

create
	make

feature {NONE} -- Initialization
	make( an_id : INTEGER; is_a_male : BOOLEAN; a_first_name, a_last_name : STRING; a_date: DATE2) is
			-- Initializes a person with gender, first- and lastname a birthdate
		require
			strings_not_void: a_first_name /= void and a_last_name /= void and a_date /= void
			strings_not_empty: not a_first_name.is_empty and not a_last_name.is_empty
			valid_date: a_date.is_valid
			positive_id: an_id > 0
		do
			id := an_id
			is_Male := is_a_male

			firstName := a_first_name
			lastName  := a_last_name
			birthName := a_last_name

			dateOfBirth := a_date

			create partners.make
			create marriagedates.make
			create divorceDates.make
			create children.make
			create adoptedchildren.make
			create adoptdates.make
			create parents.make
		end


feature -- Access
	get_id : INTEGER is
			-- returns id of the person
		do
			Result := id
		end

	get_FirstName : STRING is
			-- returns the first name
		do
			Result := firstname
		end

	get_LastName : STRING is
			-- returns the last name
		do
			Result := lastname
		end

	get_BirthName : STRING is
			-- returns the birth name
		do
			Result := birthname
		end

	get_DateOfBirth : DATE2 is
			-- returns the date of birth
		do
			Result := dateofbirth
		end

	get_DateOfDeath : DATE2 is
			-- returns the date of death
		do
			Result := dateofdeath
		end


	get_CurrentPartner : PERSON2 is
			-- returns currentPartner
		do
			Result := currentpartner
		end

	get_Partners : LIST[PERSON2] is
			-- returns list of all partners the person ever had
		do
			Result := partners.twin
		end

	get_Partner_Ids : LIST[INTEGER] is
			-- returns list of ids of all children
		do
			Result := create {LINKED_LIST[INTEGER]}.make
			from
				partners.start
			until
				partners.after
			loop
				Result.extend( partners.item.get_id )
				partners.forth
			end
		end

	get_MarriageDates : LIST[DATE2] is
			-- returns list of all marriage dates
		do
			Result := marriagedates.deep_twin
		end

	get_DivorceDates : LIST[DATE2] is
			-- returns list of all divorce dates
		do
			Result := divorcedates.deep_twin
		end

	get_Children : LIST[PERSON2] is
			-- returns list of all children
		do
			Result := children.twin
		end

	get_Children_Ids : LIST[INTEGER] is
			-- returns list of ids of all children
		do
			Result := create {LINKED_LIST[INTEGER]}.make
			from
				children.start
			until
				children.after
			loop
				Result.extend( children.item.get_id )
				children.forth
			end
		end

	get_AdoptedChildren : LIST[PERSON2] is
			-- returns list of all children
		do
			Result := adoptedChildren.twin
		end

	get_AdoptedChildren_Ids : LIST[INTEGER] is
			-- returns list of ids of all adopted children
		do
			Result := create {LINKED_LIST[INTEGER]}.make
			from
				adoptedchildren.start
			until
				adoptedchildren.after
			loop
				Result.extend( adoptedchildren.item.get_id )
				adoptedchildren.forth
			end
		end

	get_AdoptDates : LIST[DATE2] is
			-- returns list of all divorce dates
		do
			Result := adoptDates.deep_twin
		end

	get_Parents : LIST[PERSON2] is
			-- returns list of all parents
		do
			Result := parents.twin
		end

	get_Parent_Ids : LIST[INTEGER] is
			-- returns list of ids of all parents
		do
			Result := create {LINKED_LIST[INTEGER]}.make
			from
				parents.start
			until
				parents.after
			loop
				Result.extend( parents.item.get_id )
				parents.forth
			end
		end

	get_NaturalMother : PERSON2 is
			-- returns natural mother
		do
			Result := naturalmother
		end

	get_NaturalFather : PERSON2 is
			-- returns natural father
		do
			Result := naturalfather
		end

	get_NaturalParent_ids : LIST[INTEGER] is
			-- returns list with id of natural mother and natural father, if defined
		do
			Result := create {LINKED_LIST[INTEGER]}.make
			if naturalfather /= void then
				Result.extend( naturalfather.get_id )
			end
			if naturalmother /= void then
				Result.extend( naturalmother.get_id )
			end
		end


	get_CurrentMother : PERSON2 is
			-- returns current mother
		do
			Result := currentmother
		end

	get_CurrentFather : PERSON2 is
			-- returns current father
		do
			Result := currentfather
		end

	is_Male : BOOLEAN
		-- is current person male

	is_earlier_born_than( a_date : DATE2 ) : BOOLEAN is
			-- checks, if this person is earlier born than a_date
			-- by checking the birthdates of all ancestors until one is comparable
			-- since only using dates and no time, the same date will return true
			require
				valid_date: a_date /= void and a_date.is_valid
			do
				-- check if current is born later
				if dateofbirth.is_later_than( a_date ) then
					Result := false
				-- check if current is born earlier
				elseif a_date.is_later_than( dateofbirth ) then
					Result := true
				-- check if current has no parents, if not than we assume that it is ok
				elseif parents.is_empty then
					Result := true
				else
					-- make the check for all parents
					from
						parents.start
						Result := true
					until
						parents.after
					loop
						Result := Result and parents.item.is_earlier_born_than ( a_date )
						parents.forth
					end
				end
			end

	has_direct_sibling( a_person : PERSON2 ) : BOOLEAN is
		-- returns true, if a_person is a direct sibling of this
		do
			Result := false
			if naturalfather /= Void then
				Result := Result or naturalfather.get_children.has ( a_person )
			end
			if naturalmother /= Void then
				Result := Result or naturalmother.get_children.has ( a_person )
			end
		end

	has_direct_ancestor( a_person : PERSON2 ) : BOOLEAN is
		-- returns true, if a_person is a direct ancestor of this
		do
			Result := false
			if naturalfather /= Void then
				Result := (Result or (naturalfather.get_id = a_person.get_id)) or else naturalfather.has_direct_ancestor( a_person )
			end
			if naturalmother /= Void and not Result then
				Result := (Result or (naturalmother.get_id = a_person.get_id)) or else naturalmother.has_direct_ancestor( a_person )
			end
		end


feature -- Element change
	set_LastName( newLastName : STRING) is
			-- sets lastName to newLastName
		require
			newLastName_not_valid: newLastName /= void and then not newLastName.is_empty
		do
			lastName := newLastName
		end

	set_DateOfDeath( newDateOfDeath : DATE2 ) is
			-- sets dateOfDeath to newDateOfDeath
		require
			date_valid: newDateOfDeath /= void and then newDateOfDeath.is_valid
			not_dead_yet: get_dateofdeath() = Void
		do
			dateOfDeath := newDateOfDeath
		end

	marry( partner : PERSON2; a_last_name : STRING; a_date : DATE2 ) is
		-- marries this with the partner
		-- caller must make sure, that the partner is married to this too
		require
			valid_partner: partner /= Void
			valid_date: a_date /= Void and then a_date.is_valid
			not_married: (get_currentpartner = Void) or else (get_currentpartner.get_dateofdeath /= Void)
			valid_last_name: (a_last_name = Void) or else (not a_last_name.is_empty)
		local
			unknown_date : DATE2
		do
			partners.extend( partner )
			currentpartner := partner
			marriagedates.extend( a_date )
			create unknown_date.make ("??.??.????")  -- BAD IDEA!
			divorcedates.extend( unknown_date )
			if a_last_name /= Void then
				lastName := a_last_name
			end
		end

	divorce( a_last_name : STRING; a_date : DATE2 ) is
		-- divorces from current partner
		-- caller must make sure, that the partner is divorced to this too
		require
			valid_partner: get_currentpartner /= Void
			valid_date: a_date /= Void and then a_date.is_valid
			later: not get_marriagedates.last.is_later_than( a_date )
			valid_last_name: (a_last_name = Void) or else (not a_last_name.is_empty)
		do
			divorcedates.finish
			divorcedates.replace( a_date )
			currentpartner := Void
			if a_last_name /= Void then
				lastName := a_last_name
			end
		end

	addChild( newChild : PERSON2 ) is
			-- adds a newChild
		require
			child_not_void: newChild /= void
		do
			children.extend( newchild )
		end

	adoptChild( newChild : PERSON2; date : DATE2  ) is
			-- adds a newChild
		require
			child_not_void: newChild /= void
		do
			adoptedChildren.extend( newchild )
			adoptDates.extend( date )
		end

	set_NaturalMother( mother : PERSON2 ) is
			-- sets natural mother
		require
			mother_not_void: mother /= void
		do
			parents.extend( mother )
			naturalMother := mother
			currentMother := mother
		end

	set_NaturalFather( father : PERSON2 ) is
			-- sets natural father
		require
			father_not_void: father /= void
		do
			parents.extend( father )
			naturalFather := father
			currentFather := father
		end

	set_AdoptingMother( mother : PERSON2 ) is
			-- sets current mother
		require
			mother_not_void: mother /= void
		do
			parents.extend( mother )
			currentMother := mother
		end

	set_AdoptingFather( father : PERSON2 ) is
			-- sets current father
		require
			father_not_void: father /= void
		do
			parents.extend( father )
			currentFather := father
		end

feature {NONE} -- Implementation

	id			: INTEGER

	firstName 	: STRING
	lastName  	: STRING
	birthName	: STRING

	dateOfBirth : DATE2
	dateOfDeath : DATE2

	currentPartner	: PERSON2
	partners		: LINKED_LIST[PERSON2]
	marriageDates	: LINKED_LIST[DATE2]
	divorceDates	: LINKED_LIST[DATE2]

	children 		: LINKED_LIST[PERSON2]
	adoptedChildren : LINKED_LIST[PERSON2]
	adoptDates		: LINKED_LIST[DATE2]

	parents 		: LINKED_LIST[PERSON2]
	naturalMother, naturalFather : PERSON2
	currentMother, currentFather : PERSON2

invariant
	equal_number_of_partners_and_marriagedates: partners.count = marriageDates.count
	equal_number_of_partners_and_divorcedates: partners.count = divorceDates.count
	valid_birth_date: dateOfBirth.is_valid
	valid_death_date: dateOfDeath = Void or else dateOfDeath.is_valid

end
